from rocolib.utils import numsym as np


def MirrorX():
  return np.diag([-1, 1, 1, 1])

def MirrorY():
  return np.diag([1, -1, 1, 1])

def MirrorZ():
  return np.diag([1, 1, -1, 1])

def Scale(scale):
  return np.diag([scale, scale, scale, 1])

def RotateX(angle):
  r = np.array([[1, 0, 0, 0],
                [0, np.cos(angle), -np.sin(angle), 0],
                [0, np.sin(angle),  np.cos(angle), 0],
                [0, 0, 0, 1]])
  return r

def RotateY(angle):
  r = np.array([[ np.cos(angle), 0, np.sin(angle), 0],
                [0, 1, 0, 0],
                [-np.sin(angle), 0, np.cos(angle), 0],
                [0, 0, 0, 1]])
  return r

def RotateZ(angle):
  r = np.array([[np.cos(angle), -np.sin(angle), 0, 0],
                [np.sin(angle),  np.cos(angle), 0, 0],
                [0, 0, 1, 0],
                [0, 0, 0, 1]])
  return r

def MoveToOrigin(pt):
  return Translate([-pt[0], -pt[1], 0])


def RotateOntoX(pt, pt2=(0,0)):
  dx = pt[0] - pt2[0]
  dy = pt[1] - pt2[1]
  l = np.sqrt(dx * dx + dy * dy)
  dx = dx / l
  dy = dy / l
  r = np.array([[ dx,  dy, 0, 0],
                [-dy,  dx, 0, 0],
                [  0,   0, 1, 0],
                [  0,   0, 0, 1]])
  return r #RotateZ(-symbolic_atan2(pt[1] - pt2[1], pt[0] - pt2[0]))


def MoveOriginTo(pt):
  return Translate([pt[0], pt[1], 0])
  


def RotateXTo(pt, pt2=(0,0)):
  dx = pt[0] - pt2[0]
  dy = pt[1] - pt2[1]
  l = np.sqrt(dx * dx + dy * dy)
  dx = dx / l
  dy = dy / l
  r = np.array([[ dx, -dy, 0, 0],
                [ dy,  dx, 0, 0],
                [  0,   0, 1, 0],
                [  0,   0, 0, 1]])
  return r #RotateZ(symbolic_atan2(pt[1] - pt2[1], pt[0] - pt2[0]))


def Translate(origin):
  r = np.array([[1, 0, 0, origin[0]],
                [0, 1, 0, origin[1]],
                [0, 0, 1, origin[2]],
                [0, 0, 0, 1]])
  return r

def quat2DCM(quat):
  (a, b, c, d) = quat
  r = np.array([[a**2 + b**2 - c**2 - d**2, 2*b*c - 2*a*d, 2*b*d + 2*a*c, 0],
                [2*b*c + 2*a*d, a**2 - b**2 + c**2 - d**2, 2*c*d - 2*a*b, 0],
                [2*b*d - 2*a*c, 2*c*d + 2*a*b, a**2 - b**2 - c**2 + d**2, 0],
                [0, 0, 0, 1]])
  return r

def Transform6DOF(origin, euler=None, quat=None):
  transform3D = Translate(origin)
  if euler:
    transform3D = np.dot(transform3D, RotateZ(euler[2]))
    transform3D = np.dot(transform3D, RotateY(euler[1]))
    transform3D = np.dot(transform3D, RotateX(euler[0]))
  elif quat:
    transform3D = np.dot(transform3D, quat2DCM(quat))
  return transform3D

